@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.29 2002/05/30 13:00:20 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.30 2002/07/11 14:26:41 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1010,7 +1010,7 @@ ip_vs_add_service(struct ip_vs_rule_user *ur, struct ip_vs_service **svc_p)
 	svc->port = ur->vport;
 	svc->fwmark = ur->vfwmark;
 	svc->flags = ur->vs_flags;
-	svc->timeout = ur->timeout;
+	svc->timeout = ur->timeout * HZ;
 	svc->netmask = ur->netmask;
 
 	INIT_LIST_HEAD(&svc->destinations);
@@ -1092,7 +1092,7 @@ static int ip_vs_edit_service(struct ip_vs_service *svc,
 	 * Set the flags and timeout value
 	 */
 	svc->flags = ur->vs_flags | IP_VS_SVC_F_HASHED;
-	svc->timeout = ur->timeout;
+	svc->timeout = ur->timeout * HZ;
 	svc->netmask = ur->netmask;
 
 	old_sched = svc->scheduler;
@@ -1635,23 +1635,23 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
 static int ip_vs_set_timeouts(struct ip_vs_rule_user *u)
 {
 	IP_VS_DBG(2, "Setting timeout tcp:%d tcpfin:%d udp:%d\n",
-		  u->tcp_timeout/HZ,
-		  u->tcp_fin_timeout/HZ,
-		  u->udp_timeout/HZ);
+		  u->tcp_timeout,
+		  u->tcp_fin_timeout,
+		  u->udp_timeout);
 
 	if (u->tcp_timeout) {
 		vs_timeout_table.timeout[IP_VS_S_ESTABLISHED]
-			= u->tcp_timeout;
+			= u->tcp_timeout * HZ;
 	}
 
 	if (u->tcp_fin_timeout) {
 		vs_timeout_table.timeout[IP_VS_S_FIN_WAIT]
-			= u->tcp_fin_timeout;
+			= u->tcp_fin_timeout * HZ;
 	}
 
 	if (u->udp_timeout) {
 		vs_timeout_table.timeout[IP_VS_S_UDP]
-			= u->udp_timeout;
+			= u->udp_timeout * HZ;
 	}
 	return 0;
 }
@@ -1813,7 +1813,7 @@ __ip_vs_get_service_entries(const struct ip_vs_get_services *get,
 			entry.fwmark = svc->fwmark;
 			strcpy(entry.sched_name, svc->scheduler->name);
 			entry.flags = svc->flags;
-			entry.timeout = svc->timeout;
+			entry.timeout = svc->timeout / HZ;
 			entry.netmask = svc->netmask;
 			entry.num_dests = svc->num_dests;
 			__ip_vs_copy_stats(&entry.stats, &svc->stats);
@@ -1837,7 +1837,7 @@ __ip_vs_get_service_entries(const struct ip_vs_get_services *get,
 			entry.fwmark = svc->fwmark;
 			strcpy(entry.sched_name, svc->scheduler->name);
 			entry.flags = svc->flags;
-			entry.timeout = svc->timeout;
+			entry.timeout = svc->timeout / HZ;
 			entry.netmask = svc->netmask;
 			entry.num_dests = svc->num_dests;
 			__ip_vs_copy_stats(&entry.stats, &svc->stats);
@@ -1896,6 +1896,13 @@ __ip_vs_get_dest_entries(const struct ip_vs_get_dests *get,
 	return ret;
 }
 
+static inline void
+__ip_vs_get_timeouts(struct ip_vs_timeout_user *u)
+{
+	u->tcp_timeout = vs_timeout_table.timeout[IP_VS_S_ESTABLISHED] / HZ;
+	u->tcp_fin_timeout = vs_timeout_table.timeout[IP_VS_S_FIN_WAIT] / HZ;
+	u->udp_timeout = vs_timeout_table.timeout[IP_VS_S_UDP] / HZ;
+}
 
 static int
 do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
@@ -1973,7 +1980,7 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 		if (svc) {
 			strcpy(get.sched_name, svc->scheduler->name);
 			get.flags = svc->flags;
-			get.timeout = svc->timeout;
+			get.timeout = svc->timeout / HZ;
 			get.netmask = svc->netmask;
 			get.num_dests = svc->num_dests;
 			__ip_vs_copy_stats(&get.stats, &svc->stats);
@@ -2013,9 +2020,7 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 			IP_VS_ERR("length: %u < %u\n", *len, sizeof(u));
 			return -EINVAL;
 		}
-		u.tcp_timeout = vs_timeout_table.timeout[IP_VS_S_ESTABLISHED];
-		u.tcp_fin_timeout = vs_timeout_table.timeout[IP_VS_S_FIN_WAIT];
-		u.udp_timeout = vs_timeout_table.timeout[IP_VS_S_UDP];
+		__ip_vs_get_timeouts(&u);
 		if (copy_to_user(user, &u, sizeof(u)) != 0)
 			ret = -EFAULT;
 	}
